use components::admin::Admin;
use components::root::Root;
use std::rc::Rc;
use yew::prelude::*;
use yew_router::BrowserRouter;
use yew_router::Routable;
use yew_router::Switch;

mod components;
mod services;

#[derive(Debug, Clone, Copy, PartialEq, Routable)]
enum Route {
    #[at("/")]
    Root,
    #[at("/chat")]
    Admin,
    #[not_found]
    #[at("/404")]
    NotFound,
}

fn switch(selected_route: Route) -> Html {
    match selected_route {
        Route::Root => html! {<Root />},
        Route::Admin => html! {<Admin/>},
        Route::NotFound => html! {<h1>{"404 not found"}</h1>},
    }
}

#[derive(Default, Clone, Debug, PartialEq, Eq)]
struct AppStateInner {
    state: common::State,
}
type AppState = Rc<AppStateInner>;

#[function_component]
fn App() -> Html {
    // let counter = use_state(|| 0);
    // let onclick = {
    //     let counter = counter.clone();
    //     move |_| {
    //         let value = *counter + 1;
    //         counter.set(value);
    //     }
    // };

    let ctx = use_state(|| Rc::new(AppStateInner::default()));

    html! {
        <ContextProvider<AppState> context={(*ctx).clone()}>
        <BrowserRouter>
            <div class="flex w-screen h-screen">
                <Switch<Route> render={switch}/>
            </div>
        </BrowserRouter>
        </ContextProvider<AppState>>
    }
}

fn main() {
    wasm_logger::init(wasm_logger::Config::default());
    yew::Renderer::<App>::new().render();
}
